@@ -3,10 +3,10 @@
 // The code in this file is part of PyXPlot
 // <http://www.pyxplot.org.uk>
 //
-// Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
-//               2008-2010 Ross Church
+// Copyright (C) 2006-2011 Dominic Ford <coders@pyxplot.org.uk>
+//               2008-2011 Ross Church
 //
-// $Id: eps_plot_linkedaxes.c 905 2010-07-25 19:31:16Z dcf21 $
+// $Id: eps_plot_linkedaxes.c 972 2010-11-09 15:32:04Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -114,7 +114,7 @@ void eps_plot_LinkedAxisBackPropagate(EPSComm *x, settings_axis *source)
 void eps_plot_DecideAxisRanges(EPSComm *x)
  {
   int            i, j;
-  settings_axis *axes;
+  settings_axis *axes=NULL;
 
   // Decide the range of each axis in turn
   for (j=0; j<3; j++)
@@ -122,6 +122,7 @@ void eps_plot_DecideAxisRanges(EPSComm *x)
     if      (j==0) axes = x->current->XAxes;
     else if (j==1) axes = x->current->YAxes;
     else if (j==2) axes = x->current->ZAxes;
+    else           ppl_fatal(__FILE__,__LINE__,"Internal fail.");
     for (i=0; i<MAX_AXES; i++)
      {
       if (!axes[i].RangeFinalised   ) { eps_plot_LinkedAxisForwardPropagate(x, &axes[i], 1); if (*x->status) return; }
@@ -263,18 +264,7 @@ int eps_plot_LinkedAxisLinkUsing(settings_axis *out, settings_axis *in)
   else                 VarName = "c";
 
   // Look up variable in user space and get pointer to its value
-  DictLookup(_ppl_UserSpace_Vars, VarName, NULL, (void **)&VarVal);
-  if (VarVal!=NULL)
-   {
-    DummyTemp = *VarVal;
-   }
-  else
-   {
-    ppl_units_zero(&DummyTemp);
-    DictAppendValue(_ppl_UserSpace_Vars, VarName, DummyTemp);
-    DictLookup(_ppl_UserSpace_Vars, VarName, NULL, (void **)&VarVal);
-    DummyTemp.modified = 2;
-   }
+  ppl_UserSpace_GetVarPointer(VarName, &VarVal, &DummyTemp);
 
   out->AxisLinearInterpolation = (double *)lt_malloc(AXISLINEARINTERPOLATION_NPOINTS * sizeof(double));
   out->AxisValueTurnings       = 0;
@@ -316,7 +306,7 @@ int eps_plot_LinkedAxisLinkUsing(settings_axis *out, settings_axis *in)
    }
 
   // Restore original value of x (or y/z)
-  *VarVal = DummyTemp;
+  ppl_UserSpace_RestoreVarPointer(&VarVal, &DummyTemp);
   out->AxisTurnings[xrn--] = AXISLINEARINTERPOLATION_NPOINTS-1;
   out->AxisValueTurnings = xrn;
   return 0;
@@ -449,18 +439,7 @@ void eps_plot_LinkUsingBackPropagate(EPSComm *x, double val, settings_axis *targ
   else                     VarName = "c";
 
   // Look up xyz dummy variable in user space and get pointer to its value
-  DictLookup(_ppl_UserSpace_Vars, VarName, NULL, (void **)&VarVal);
-  if (VarVal!=NULL)
-   {
-    DummyTemp = *VarVal;
-   }
-  else
-   {
-    ppl_units_zero(&DummyTemp);
-    DictAppendValue(_ppl_UserSpace_Vars, VarName, DummyTemp);
-    DictLookup(_ppl_UserSpace_Vars, VarName, NULL, (void **)&VarVal);
-    DummyTemp.modified = 2;
-   }
+  ppl_UserSpace_GetVarPointer(VarName, &VarVal, &DummyTemp);
 
   // Set up commlink structure to use in minimiser
   commlink.expr        = source->linkusing;
@@ -530,7 +509,7 @@ void eps_plot_LinkUsingBackPropagate(EPSComm *x, double val, settings_axis *targ
    }
 
   // Restore original value of x (or y/z)
-  *VarVal = DummyTemp;
+  ppl_UserSpace_RestoreVarPointer(&VarVal, &DummyTemp);
   return;
  }
 
