@@ -3,10 +3,10 @@
 // The code in this file is part of PyXPlot
 // <http://www.pyxplot.org.uk>
 //
-// Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
-//               2008-2010 Ross Church
+// Copyright (C) 2006-2011 Dominic Ford <coders@pyxplot.org.uk>
+//               2008-2011 Ross Church
 //
-// $Id: eps_plot_styles.c 913 2010-07-29 14:10:38Z dcf21 $
+// $Id: eps_plot_styles.c 994 2010-12-27 23:15:08Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -491,9 +491,6 @@ int  eps_plot_dataset(EPSComm *x, DataTable *data, int style, unsigned char Thre
 
   if ((style == SW_STYLE_POINTS) || (style == SW_STYLE_LINESPOINTS) || (style == SW_STYLE_STARS) || (style == SW_STYLE_DOTS)) // POINTS, DOTS, STARS
    {
-    double final_pointsize = pd->ww_final.pointsize;
-    if      (style == SW_STYLE_DOTS ) final_pointsize *=  0.05; // Dots are 1/20th size of points
-    else if (style == SW_STYLE_STARS) final_pointsize *= 12.0 ; // Stars are BIG
     last_colstr=NULL;
 
     blk = data->first;
@@ -501,6 +498,7 @@ int  eps_plot_dataset(EPSComm *x, DataTable *data, int style, unsigned char Thre
      {
       for (j=0; j<blk->BlockPosition; j++)
        {
+        double final_pointsize=0.0;
         eps_plot_GetPosition(&xpos, &ypos, &depth, &xap, &yap, &zap, NULL, NULL, NULL, ThreeDim, UUR(xn), UUR(yn), ThreeDim ? UUR(zn) : 0.0, a[xn], a[yn], a[zn], xrn, yrn, zrn, sg, origin_x, origin_y, scale_x, scale_y, scale_z, 0);
         if (!gsl_finite(xpos)) // Position of point is off side of graph
          {
@@ -510,13 +508,16 @@ int  eps_plot_dataset(EPSComm *x, DataTable *data, int style, unsigned char Thre
 
         // Work out style information for next point
         eps_plot_WithWordsFromUsingItems(&pd->ww_final, &blk->data_real[Ncolumns*j].d, Ncolumns);
+        final_pointsize = pd->ww_final.pointsize;
+        if      (style == SW_STYLE_DOTS ) final_pointsize *=  0.05; // Dots are 1/20th size of points
+        else if (style == SW_STYLE_STARS) final_pointsize *= 12.0 ; // Stars are BIG
         eps_core_SetColour(x, &pd->ww_final, 0);
         IF_NOT_INVISIBLE
          {
           if ((last_colstr==NULL)||(strcmp(last_colstr,x->CurrentColour)!=0)) { last_colstr = (char *)lt_malloc(strlen(x->CurrentColour)+1); if (last_colstr==NULL) break; strcpy(last_colstr, x->CurrentColour); }
           if (style != SW_STYLE_STARS)
            {
-            pt = (style == SW_STYLE_DOTS) ? 9 : ((pd->ww_final.pointtype-1) % N_POINTTYPES); // Dots are always pt 9 (circle)
+            pt = (style == SW_STYLE_DOTS) ? 16 : ((pd->ww_final.pointtype-1) % N_POINTTYPES); // Dots are always pt 17 (filled circle)
             while (pt<0) pt+=N_POINTTYPES;
             x->PointTypesUsed[pt] = 1;
             sprintf(epsbuff, "%.2f %.2f pt%d", xpos, ypos, pt+1);
@@ -1002,8 +1003,8 @@ int  eps_plot_dataset(EPSComm *x, DataTable *data, int style, unsigned char Thre
   else if ((style == SW_STYLE_SURFACE)&&(xrn==0)&&(yrn==0)&&(zrn==0)) // SURFACE
    {
     int  fill;
-    long XSize = (x->current->settings.SamplesXAuto==SW_BOOL_TRUE) ? x->current->settings.samples : x->current->settings.SamplesX;
-    long YSize = (x->current->settings.SamplesYAuto==SW_BOOL_TRUE) ? x->current->settings.samples : x->current->settings.SamplesY;
+    int  XSize = pd->GridXSize;
+    int  YSize = pd->GridYSize;
     long X,Y; long j;
     double xap,yap,zap,theta_x,theta_y,theta_z,depth;
     double x1,y1,z1,x2,y2,z2,x3,y3,z3,x4,y4,z4;
@@ -1013,7 +1014,7 @@ int  eps_plot_dataset(EPSComm *x, DataTable *data, int style, unsigned char Thre
    if ((xap<0)||(xap>1)||(yap<0)||(yap>1)||(zap<0)||(zap>1)) continue; \
    if ( (!gsl_finite(XO))||(!gsl_finite(YO))||(!gsl_finite(ZO)) ) continue;
 
-    if (blk->BlockPosition != XSize * YSize)
+    if (blk->BlockPosition != ((long)XSize) * YSize)
      {
       ppl_error(ERR_INTERNAL,-1,-1,"Surface plot yielded data grid of the wrong size.");
      }
@@ -1024,7 +1025,7 @@ int  eps_plot_dataset(EPSComm *x, DataTable *data, int style, unsigned char Thre
        for (Y=0; Y<YSize-1; Y++)
         for (X=0; X<XSize-1; X++)
          {
-          j = X + Y*XSize;
+          j = X + Y*((long)XSize);
           eps_plot_WithWordsFromUsingItems(&pd->ww_final, &blk->data_real[Ncolumns*j].d, Ncolumns); // Work out style information for next point
           if (fill)
            {
@@ -1092,6 +1093,7 @@ void eps_plot_LegendIcon(EPSComm *x, int i, canvas_plotdesc *pd, double xpos, do
     double final_pointsize = pd->ww_final.pointsize;
     if (style==SW_STYLE_DOTS) final_pointsize *= 0.05; // Dots are 1/20th size of points
     eps_core_SetColour(x, &pd->ww_final, 1);
+    eps_core_SetLinewidth(x, EPS_DEFAULT_LINEWIDTH * pd->ww_final.pointlinewidth, 1, 0);
     IF_NOT_INVISIBLE
      {
       int pt = (pd->ww_final.pointtype-1) % N_POINTTYPES;
