@@ -3,10 +3,10 @@
 // The code in this file is part of PyXPlot
 // <http://www.pyxplot.org.uk>
 //
-// Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
-//               2008-2010 Ross Church
+// Copyright (C) 2006-2011 Dominic Ford <coders@pyxplot.org.uk>
+//               2008-2011 Ross Church
 //
-// $Id: ppl_canvasdraw.c 953 2010-09-09 16:11:57Z dcf21 $
+// $Id: ppl_canvasdraw.c 972 2010-11-09 15:32:04Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -141,7 +141,7 @@ static void(*AfterHandlers[])(EPSComm *) = {NULL                       , NULL
 // Main entry point for rendering a canvas to graphical output
 void canvas_draw(unsigned char *unsuccessful_ops)
  {
-  int i, j, termtype, status=0, CSPCommand;
+  int i, j, termtype, status=0, CSPCommand=0;
   static long TempFile_counter=0, TeXFile_counter=0;
   char EPSFilenameTemp[FNAME_LENGTH], TeXFilenameTemp[FNAME_LENGTH], TitleTemp[FNAME_LENGTH], FinalFilenameTemp[FNAME_LENGTH], GSOutputTemp[FNAME_LENGTH];
   wordexp_t WordExp;
