@@ -3,10 +3,10 @@
 // The code in this file is part of PyXPlot
 // <http://www.pyxplot.org.uk>
 //
-// Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
-//               2008-2010 Ross Church
+// Copyright (C) 2006-2011 Dominic Ford <coders@pyxplot.org.uk>
+//               2008-2011 Ross Church
 //
-// $Id: ppl_canvasitems.c 905 2010-07-25 19:31:16Z dcf21 $
+// $Id: ppl_canvasitems.c 974 2010-11-13 20:58:47Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -25,6 +25,9 @@
 #include <stdio.h>
 #include <string.h>
 
+#include "EPSMaker/eps_plot.h"
+#include "EPSMaker/eps_plot_styles.h"
+
 #include "ListTools/lt_dict.h"
 #include "ListTools/lt_memory.h"
 
@@ -431,6 +434,7 @@ char *canvas_item_textify(canvas_item *ptr, char *output)
       if (pd_first) { pd_first=0; } else { output[i++]=','; }
       if (pd->parametric) { sprintf(output+i, " parametric"); i+=strlen(output+i); }
       if (pd->TRangeSet)  { sprintf(output+i, " [%s:%s]", ppl_units_NumericDisplay(&pd->Tmin,0,0,0), ppl_units_NumericDisplay(&pd->Tmax,1,0,0)); i+=strlen(output+i); }
+      if (pd->VRangeSet)  { sprintf(output+i, " [%s:%s]", ppl_units_NumericDisplay(&pd->Vmin,0,0,0), ppl_units_NumericDisplay(&pd->Vmax,1,0,0)); i+=strlen(output+i); }
       if (!pd->function) { output[i++]=' '; StrEscapify(pd->filename, output+i); i+=strlen(output+i); } // Filename of datafile we are plotting
       else
        for (j=0; j<pd->NFunctions; j++) // Print out the list of functions which we are plotting
@@ -1155,9 +1159,10 @@ int directive_piechart(Dict *command, int interactive)
   ptr->plotitems=(canvas_plotdesc *)malloc(sizeof(canvas_plotdesc));
   if (ptr->plotitems == NULL) { ppl_error(ERR_MEMORY, -1, -1,"Out of memory."); return 1; }
   memset((void *)(ptr->plotitems), 0, sizeof(canvas_plotdesc));
+  ptr->plotitems->PersistentDataTable=NULL;
 
   // Store either filename of datafile, or the list of functions which we are plotting
-  DictLookup(command, "filename", NULL, (void **)&tempstr);
+  DictLookup(command, "filename", NULL, (void *)&tempstr);
   if (tempstr != NULL) // We are plotting a datafile
    {
     ptr->plotitems->function = 0;
@@ -1170,7 +1175,7 @@ int directive_piechart(Dict *command, int interactive)
     ptr->plotitems->function = 1;
     ptr->plotitems->filename = NULL;
     ptr->plotitems->parametric = 0;
-    DictLookup(command, "expression_list:", NULL, (void **)&ExpressionList);
+    DictLookup(command, "expression_list:", NULL, (void *)&ExpressionList);
     j = ptr->plotitems->NFunctions = ListLen(ExpressionList);
     ptr->plotitems->functions = (char **)malloc(j * sizeof(char *));
     if (ptr->plotitems->functions == NULL) { ppl_error(ERR_MEMORY, -1, -1,"Out of memory."); free(ptr->plotitems); ptr->plotitems = NULL; return 1; }
@@ -1178,7 +1183,7 @@ int directive_piechart(Dict *command, int interactive)
     for (i=0; i<j; i++)
      {
       TempDict2 = (Dict *)ListIter2->data;
-      DictLookup(TempDict2, "expression", NULL, (void **)&tempstr);
+      DictLookup(TempDict2, "expression", NULL, (void *)&tempstr);
       ptr->plotitems->functions[i] = (char *)malloc(strlen(tempstr)+1);
       if (ptr->plotitems->functions[i] == NULL) { ppl_error(ERR_MEMORY, -1, -1,"Out of memory."); free(ptr->plotitems); ptr->plotitems = NULL; return 1; }
       strcpy(ptr->plotitems->functions[i], tempstr);
@@ -1187,7 +1192,7 @@ int directive_piechart(Dict *command, int interactive)
    }
 
   // Loop up format string
-  DictLookup(command, "format_string", NULL, (void **)&tempstr);
+  DictLookup(command, "format_string", NULL, (void *)&tempstr);
   if (tempstr==NULL) { ptr->text = NULL; }
   else
    {
@@ -1197,7 +1202,7 @@ int directive_piechart(Dict *command, int interactive)
    }
 
   // Look up label to apply to datapoints
-  DictLookup(command, "label", NULL, (void **)&tempstr);
+  DictLookup(command, "label", NULL, (void *)&tempstr);
   if (tempstr==NULL) { ptr->plotitems->label = NULL; }
   else
    {
@@ -1207,18 +1212,18 @@ int directive_piechart(Dict *command, int interactive)
    }
 
   // Look up label position
-  DictLookup(command, "piekeypos", NULL, (void **)&tempstr);
+  DictLookup(command, "piekeypos", NULL, (void *)&tempstr);
   if (tempstr==NULL) ptr->ArrowType = SW_PIEKEYPOS_AUTO;
   else               ptr->ArrowType = FetchSettingByName(tempstr, SW_PIEKEYPOS_INT, SW_PIEKEYPOS_STR);
 
   // Look up index , every, using modifiers
   ptr->plotitems->UsingRowCols = DATAFILE_COL;
   ptr->plotitems->index = -1;
-  DictLookup(command, "index"      , NULL, (void **)&indexptr);   if ((ptr->plotitems->IndexSet = (indexptr!=NULL))==1) ptr->plotitems->index = *indexptr;
-  DictLookup(command, "use_rows"   , NULL, (void **)&tempstr);    if (tempstr  != NULL) ptr->plotitems->UsingRowCols = DATAFILE_ROW;
-  DictLookup(command, "use_cols"   , NULL, (void **)&tempstr);    if (tempstr  != NULL) ptr->plotitems->UsingRowCols = DATAFILE_COL;
+  DictLookup(command, "index"      , NULL, (void *)&indexptr);   if ((ptr->plotitems->IndexSet = (indexptr!=NULL))==1) ptr->plotitems->index = *indexptr;
+  DictLookup(command, "use_rows"   , NULL, (void *)&tempstr);    if (tempstr  != NULL) ptr->plotitems->UsingRowCols = DATAFILE_ROW;
+  DictLookup(command, "use_cols"   , NULL, (void *)&tempstr);    if (tempstr  != NULL) ptr->plotitems->UsingRowCols = DATAFILE_COL;
 
-  DictLookup(command, "every_list:", NULL, (void **)&EveryList);
+  DictLookup(command, "every_list:", NULL, (void *)&EveryList);
   if (EveryList==NULL) { ptr->plotitems->EverySet = 0; }
   else
    {
@@ -1228,7 +1233,7 @@ int directive_piechart(Dict *command, int interactive)
     for (i=0; i<j; i++)
      {
       TempDict2 = (Dict *)ListIter2->data;
-      DictLookup(TempDict2, "every_item", NULL, (void **)&tempint);
+      DictLookup(TempDict2, "every_item", NULL, (void *)&tempint);
       if (tempint != NULL) ptr->plotitems->EveryList[i] = *tempint;
       else                 ptr->plotitems->EveryList[i] = -1;
       ListIter2 = ListIterate(ListIter2, NULL);
@@ -1236,7 +1241,7 @@ int directive_piechart(Dict *command, int interactive)
    }
 
   // Look up select modifier
-  DictLookup(command, "select_criterion", NULL, (void **)&SelectCrit);
+  DictLookup(command, "select_criterion", NULL, (void *)&SelectCrit);
   if (SelectCrit==NULL) { ptr->plotitems->SelectCriterion = NULL; }
   else
    {
@@ -1246,7 +1251,7 @@ int directive_piechart(Dict *command, int interactive)
    }
 
   // Look up using modifiers
-  DictLookup(command, "using_list:", NULL, (void **)&UsingList);
+  DictLookup(command, "using_list:", NULL, (void *)&UsingList);
   if (UsingList==NULL) { ptr->plotitems->NUsing = 0; ptr->plotitems->UsingList = NULL; }
   else
    {
@@ -1255,7 +1260,7 @@ int directive_piechart(Dict *command, int interactive)
      {
       TempDict2 = (Dict *)(UsingList->first->data);
       if (TempDict2==NULL) { ptr->plotitems->NUsing = 0; ptr->plotitems->UsingList = NULL; goto FinishedUsing; }
-      DictLookup(TempDict2, "using_item", NULL, (void **)&cptr2);
+      DictLookup(TempDict2, "using_item", NULL, (void *)&cptr2);
       if (cptr2==NULL) { ptr->plotitems->NUsing = 0; ptr->plotitems->UsingList = NULL; goto FinishedUsing; }
      }
     ptr->plotitems->NUsing = j;
@@ -1265,7 +1270,7 @@ int directive_piechart(Dict *command, int interactive)
     for (i=0; i<j; i++)
      {
       TempDict2 = (Dict *)ListIter2->data;
-      DictLookup(TempDict2, "using_item", NULL, (void **)&cptr2);
+      DictLookup(TempDict2, "using_item", NULL, (void *)&cptr2);
       if (cptr2==NULL) cptr2=""; // NULL expression means blank using expression
       ptr->plotitems->UsingList[i] = (char *)malloc(strlen(cptr2)+1);
       if (ptr->plotitems->UsingList[i] == NULL)  { ppl_error(ERR_MEMORY, -1, -1,"Out of memory."); free(ptr->plotitems); ptr->plotitems = NULL; return 1; }
@@ -1278,6 +1283,28 @@ FinishedUsing:
   // Read in style information from with clause
   with_words_fromdict(command, &ptr->plotitems->ww, 1);
 
+  // If plotting data from pipe, read and store it now
+  if ((ptr->plotitems->filename!=NULL) && ((strcmp(ptr->plotitems->filename,"-")==0) || (strcmp(ptr->plotitems->filename,"--")==0)))
+   {
+    canvas_plotdesc *pd = ptr->plotitems;
+    int              j, status=0, ErrCount, NExpect;
+    List            *UsingList, *EveryList;
+    Dict            *tempdict;
+    unsigned char    AutoUsingList=0;
+    char            *errbuff;
+
+    UsingList   = ListInit(); for (j=0; j<pd->NUsing  ; j++) { tempdict = DictInit(HASHSIZE_SMALL); DictAppendPtr(tempdict, "using_item", (void *)pd->UsingList[j], 0, 0, DATATYPE_VOID); ListAppendPtr(UsingList, (void *)tempdict, 0, 0, DATATYPE_VOID); }
+    EveryList   = ListInit(); for (j=0; j<pd->EverySet; j++) { tempdict = DictInit(HASHSIZE_SMALL); DictAppendPtr(tempdict, "every_item", (void *)(pd->EveryList+j), 0, 0, DATATYPE_VOID); ListAppendPtr(EveryList, (void *)tempdict, 0, 0, DATATYPE_VOID); }
+    ErrCount    = DATAFILE_NERRS;
+    NExpect     = 1;
+
+    errbuff = (char *)lt_malloc(LSTR_LENGTH);
+    if (errbuff==NULL) { canvas_delete(id); ppl_error(ERR_GENERAL, -1, -1, "Piechart has been removed from multiplot, because it generated an error."); return 1; }
+
+    if (eps_plot_AddUsingItemsForWithWords(&pd->ww_final, &NExpect, &AutoUsingList, UsingList)) { canvas_delete(id); ppl_error(ERR_GENERAL, -1, -1, "Piechart has been removed from multiplot, because it generated an error."); return 1; } // Add extra using items for, e.g. "linewidth $3".
+    DataFile_read(&pd->PersistentDataTable, &status, errbuff, pd->filename, pd->index, pd->UsingRowCols, UsingList, AutoUsingList, EveryList, pd->label, NExpect, pd->SelectCriterion, pd->continuity, NULL, -1, 1, &ErrCount);
+   }
+
   // Redisplay the canvas as required
   if (settings_term_current.display == SW_ONOFF_ON)
    {
@@ -1362,9 +1389,9 @@ int directive_text(Dict *command, int interactive)
   if (canvas_itemlist_add(command,CANVAS_TEXT,&ptr,&id,0)) { ppl_error(ERR_MEMORY, -1, -1,"Out of memory."); free(text); return 1; }
 
   // Check for halign or valign modifiers
-  DictLookup(command,"halign",NULL,(void **)&tempstr);
+  DictLookup(command,"halign",NULL,(void *)&tempstr);
   if (tempstr != NULL) ptr->settings.TextHAlign = FetchSettingByName(tempstr, SW_HALIGN_INT, SW_HALIGN_STR);
-  DictLookup(command,"valign",NULL,(void **)&tempstr);
+  DictLookup(command,"valign",NULL,(void *)&tempstr);
   if (tempstr != NULL) ptr->settings.TextVAlign = FetchSettingByName(tempstr, SW_VALIGN_INT, SW_VALIGN_STR);
 
   if (x  !=NULL) { ptr->xpos     = x  ->real; } else { ptr->xpos      = settings_graph_current.OriginX.real; }
@@ -1474,7 +1501,7 @@ int directive_plot(Dict *command, int interactive, int replot)
 
   if (replot) // If replotting, find canvas item to append plot items onto
    {
-    DictLookup(command, "editno", NULL, (void **)&EditNo);
+    DictLookup(command, "editno", NULL, (void *)&EditNo);
     if (EditNo == NULL) EditNo=&ReplotFocus;
     ptr = NULL;
     if (canvas_items != NULL)
@@ -1528,13 +1555,13 @@ int directive_plot(Dict *command, int interactive, int replot)
 
   if (!replot)
    {
-    DictLookup(command, "threedim", NULL, (void **)&cptr); // Set 3d flag
+    DictLookup(command, "threedim", NULL, (void *)&cptr); // Set 3d flag
     ptr->ThreeDim = (cptr!=NULL);
    }
 
   // Read list of ranges
-  DictLookup(command, "directive", NULL, (void **)&cptr);
-  DictLookup(command, "range_list", NULL, (void **)&RangeList);
+  DictLookup(command, "directive", NULL, (void *)&cptr);
+  DictLookup(command, "range_list", NULL, (void *)&RangeList);
   ListIter = ListIterateInit(RangeList);
   RangePtr = &ptr->plotranges;
   i=0;
@@ -1542,10 +1569,10 @@ int directive_plot(Dict *command, int interactive, int replot)
    {
     if (*RangePtr == NULL) { *RangePtr=(canvas_plotrange *)malloc(sizeof(canvas_plotrange)); if (*RangePtr == NULL) { ppl_error(ERR_MEMORY, -1, -1,"Out of memory."); return 1; } ppl_units_zero(&(*RangePtr)->unit); (*RangePtr)->min=(*RangePtr)->max=0.0; (*RangePtr)->MinSet=(*RangePtr)->MaxSet=(*RangePtr)->AutoMinSet=(*RangePtr)->AutoMaxSet=0; (*RangePtr)->next=NULL; }
     TempDict = (Dict *)ListIter->data;
-    DictLookup(TempDict,"min"    ,NULL,(void **)&min);
-    DictLookup(TempDict,"max"    ,NULL,(void **)&max);
-    DictLookup(TempDict,"minauto",NULL,(void **)&MinAuto);
-    DictLookup(TempDict,"maxauto",NULL,(void **)&MaxAuto);
+    DictLookup(TempDict,"min"    ,NULL,(void *)&min);
+    DictLookup(TempDict,"max"    ,NULL,(void *)&max);
+    DictLookup(TempDict,"minauto",NULL,(void *)&MinAuto);
+    DictLookup(TempDict,"maxauto",NULL,(void *)&MaxAuto);
     if ((min!=NULL)&&(max!=NULL)&&(!ppl_units_DimEqual(min,max))) { sprintf(temp_err_string, "The minimum and maximum limits specified in range %ld in the %s command have conflicting physical dimensions. The former has units of <%s>, whilst the latter has units of <%s>.", i+1, cptr, ppl_units_GetUnitStr(min,NULL,NULL,0,1,0), ppl_units_GetUnitStr(max,NULL,NULL,1,1,0)); ppl_error(ERR_NUMERIC, -1, -1, temp_err_string); return 1; }
     if ((min!=NULL)&&(max==NULL)&&(MaxAuto==NULL)&&((*RangePtr)->MaxSet)&&(!ppl_units_DimEqual(min,&(*RangePtr)->unit))) { sprintf(temp_err_string, "The minimum limit specified in range %ld in the %s command has conflicting physical dimensions with the pre-existing maximum limit set for this range. The former has units of <%s>, whilst the latter has units of <%s>.", i+1, cptr, ppl_units_GetUnitStr(min,NULL,NULL,0,1,0), ppl_units_GetUnitStr(&(*RangePtr)->unit,NULL,NULL,1,1,0)); ppl_error(ERR_NUMERIC, -1, -1, temp_err_string); return 1; }
     if ((max!=NULL)&&(min==NULL)&&(MinAuto==NULL)&&((*RangePtr)->MinSet)&&(!ppl_units_DimEqual(max,&(*RangePtr)->unit))) { sprintf(temp_err_string, "The maximum limit specified in range %ld in the %s command has conflicting physical dimensions with the pre-existing minimum limit set for this range. The former has units of <%s>, whilst the latter has units of <%s>.", i+1, cptr, ppl_units_GetUnitStr(max,NULL,NULL,0,1,0), ppl_units_GetUnitStr(&(*RangePtr)->unit,NULL,NULL,1,1,0)); ppl_error(ERR_NUMERIC, -1, -1, temp_err_string); return 1; }
@@ -1559,7 +1586,7 @@ int directive_plot(Dict *command, int interactive, int replot)
    }
 
   // Read plot items
-  DictLookup(command, "plot_list,", NULL, (void **)&PlotList);
+  DictLookup(command, "plot_list,", NULL, (void *)&PlotList);
   ListIter = ListIterateInit(PlotList);
   PlotItemPtr = &ptr->plotitems;
   while (*PlotItemPtr != NULL) PlotItemPtr=&(*PlotItemPtr)->next; // Find end of list of plot items
@@ -1568,9 +1595,9 @@ int directive_plot(Dict *command, int interactive, int replot)
     TempDict = (Dict *)ListIter->data;
 
     // Check that axes are specified in the correct format before we malloc anything
-    DictLookup(TempDict, "axis_1", NULL, (void **)&tempstr );
-    DictLookup(TempDict, "axis_2", NULL, (void **)&tempstr2);
-    DictLookup(TempDict, "axis_3", NULL, (void **)&tempstr3);
+    DictLookup(TempDict, "axis_1", NULL, (void *)&tempstr );
+    DictLookup(TempDict, "axis_2", NULL, (void *)&tempstr2);
+    DictLookup(TempDict, "axis_3", NULL, (void *)&tempstr3);
     if (!((tempstr==NULL)&&(tempstr2==NULL)&&(tempstr3==NULL)))
      {
       temp_err_string[0]='\0';
@@ -1591,7 +1618,7 @@ int directive_plot(Dict *command, int interactive, int replot)
      }
 
     // Test whether we're plotting datafile or functions
-    DictLookup(TempDict, "filename", NULL, (void **)&tempstr);
+    DictLookup(TempDict, "filename", NULL, (void *)&tempstr);
     if (tempstr != NULL) PlottingDatafiles=1; else PlottingDatafiles=0;
     glob_handle = NULL;
 
@@ -1613,6 +1640,7 @@ int directive_plot(Dict *command, int interactive, int replot)
 
       // Malloc a structure to hold this plot item
       if (*PlotItemPtr == NULL) { *PlotItemPtr=(canvas_plotdesc *)malloc(sizeof(canvas_plotdesc)); if (*PlotItemPtr == NULL) { ppl_error(ERR_MEMORY, -1, -1,"Out of memory."); return 1; } memset((void *)(*PlotItemPtr), 0, sizeof(canvas_plotdesc)); }
+      (*PlotItemPtr)->PersistentDataTable = NULL;
 
       // Enter details of datafile filename or functions into plotdesc structure
       if (PlottingDatafiles) // We are plotting a datafile
@@ -1626,12 +1654,12 @@ int directive_plot(Dict *command, int interactive, int replot)
        {
         (*PlotItemPtr)->function = 1;
         (*PlotItemPtr)->filename = NULL;
-        DictLookup(TempDict, "parametric", NULL, (void **)&tempstr);
+        DictLookup(TempDict, "parametric", NULL, (void *)&tempstr);
         (*PlotItemPtr)->parametric = (tempstr!=NULL);
-        DictLookup(TempDict, "tmin", NULL, (void **)&tempval);
+        DictLookup(TempDict, "tmin", NULL, (void *)&tempval);
         (*PlotItemPtr)->TRangeSet  = (tempval!=NULL);
         if (tempval!=NULL) (*PlotItemPtr)->Tmin = *tempval;
-        DictLookup(TempDict, "tmax", NULL, (void **)&tempval);
+        DictLookup(TempDict, "tmax", NULL, (void *)&tempval);
         if (tempval!=NULL)
          {
           (*PlotItemPtr)->Tmax = *tempval;
@@ -1639,7 +1667,18 @@ int directive_plot(Dict *command, int interactive, int replot)
           else if (!gsl_finite((*PlotItemPtr)->Tmax.real)) { sprintf(temp_err_string, "Upper limit specified for parameter t is not finite."); ppl_error(ERR_NUMERIC, -1, -1, temp_err_string); (*PlotItemPtr)->TRangeSet = 0; }
           else if (!ppl_units_DimEqual(&(*PlotItemPtr)->Tmin, &(*PlotItemPtr)->Tmax)) { sprintf(temp_err_string, "Upper and lower limits specified for parameter t have conflicting physical units of <%s> and <%s>.", ppl_units_GetUnitStr(&(*PlotItemPtr)->Tmin, NULL, NULL, 0, 1, 0), ppl_units_GetUnitStr(&(*PlotItemPtr)->Tmax, NULL, NULL, 1, 1, 0)); ppl_error(ERR_NUMERIC, -1, -1, temp_err_string); (*PlotItemPtr)->TRangeSet = 0; }
          }
-        DictLookup(TempDict, "expression_list:", NULL, (void **)&ExpressionList);
+        DictLookup(TempDict, "vmin", NULL, (void *)&tempval);
+        (*PlotItemPtr)->VRangeSet  = (tempval!=NULL);
+        if (tempval!=NULL) (*PlotItemPtr)->Vmin = *tempval;
+        DictLookup(TempDict, "vmax", NULL, (void *)&tempval);
+        if (tempval!=NULL)
+         {
+          (*PlotItemPtr)->Vmax = *tempval;
+          if      (!gsl_finite((*PlotItemPtr)->Vmin.real)) { sprintf(temp_err_string, "Lower limit specified for parameter v is not finite."); ppl_error(ERR_NUMERIC, -1, -1, temp_err_string); (*PlotItemPtr)->VRangeSet = 0; }
+          else if (!gsl_finite((*PlotItemPtr)->Vmax.real)) { sprintf(temp_err_string, "Upper limit specified for parameter v is not finite."); ppl_error(ERR_NUMERIC, -1, -1, temp_err_string); (*PlotItemPtr)->VRangeSet = 0; }
+          else if (!ppl_units_DimEqual(&(*PlotItemPtr)->Vmin, &(*PlotItemPtr)->Vmax)) { sprintf(temp_err_string, "Upper and lower limits specified for parameter v have conflicting physical units of <%s> and <%s>.", ppl_units_GetUnitStr(&(*PlotItemPtr)->Vmin, NULL, NULL, 0, 1, 0), ppl_units_GetUnitStr(&(*PlotItemPtr)->Vmax, NULL, NULL, 1, 1, 0)); ppl_error(ERR_NUMERIC, -1, -1, temp_err_string); (*PlotItemPtr)->VRangeSet = 0; }
+         }
+        DictLookup(TempDict, "expression_list:", NULL, (void *)&ExpressionList);
         j = (*PlotItemPtr)->NFunctions = ListLen(ExpressionList);
         (*PlotItemPtr)->functions = (char **)malloc(j * sizeof(char *));
         if ((*PlotItemPtr)->functions == NULL) { ppl_error(ERR_MEMORY, -1, -1,"Out of memory."); free(*PlotItemPtr); *PlotItemPtr = NULL; return 1; }
@@ -1647,7 +1686,7 @@ int directive_plot(Dict *command, int interactive, int replot)
         for (i=0; i<j; i++)
          {
           TempDict2 = (Dict *)ListIter2->data;
-          DictLookup(TempDict2, "expression", NULL, (void **)&tempstr);
+          DictLookup(TempDict2, "expression", NULL, (void *)&tempstr);
           (*PlotItemPtr)->functions[i] = (char *)malloc(strlen(tempstr)+1);
           if ((*PlotItemPtr)->functions[i] == NULL) { ppl_error(ERR_MEMORY, -1, -1,"Out of memory."); free(*PlotItemPtr); *PlotItemPtr = NULL; return 1; }
           strcpy((*PlotItemPtr)->functions[i], tempstr);
@@ -1661,15 +1700,15 @@ int directive_plot(Dict *command, int interactive, int replot)
       (*PlotItemPtr)->axis1xyz = 0;
       (*PlotItemPtr)->axis2xyz = 1;
       (*PlotItemPtr)->axis3xyz = 2;
-      DictLookup(TempDict, "axis_1", NULL, (void **)&tempstr);
+      DictLookup(TempDict, "axis_1", NULL, (void *)&tempstr);
       if (tempstr!=NULL) { (*PlotItemPtr)->axis1set = 1; (*PlotItemPtr)->axis1 = GetFloat(tempstr+1,NULL); (*PlotItemPtr)->axis1xyz = tempstr[0]-'x'; }
-      DictLookup(TempDict, "axis_2", NULL, (void **)&tempstr);
+      DictLookup(TempDict, "axis_2", NULL, (void *)&tempstr);
       if (tempstr!=NULL) { (*PlotItemPtr)->axis2set = 1; (*PlotItemPtr)->axis2 = GetFloat(tempstr+1,NULL); (*PlotItemPtr)->axis2xyz = tempstr[0]-'x'; }
-      DictLookup(TempDict, "axis_3", NULL, (void **)&tempstr);
+      DictLookup(TempDict, "axis_3", NULL, (void *)&tempstr);
       if (tempstr!=NULL) { (*PlotItemPtr)->axis3set = 1; (*PlotItemPtr)->axis3 = GetFloat(tempstr+1,NULL); (*PlotItemPtr)->axis3xyz = tempstr[0]-'x'; }
 
       // Look up label to apply to datapoints
-      DictLookup(TempDict, "label", NULL, (void **)&tempstr);
+      DictLookup(TempDict, "label", NULL, (void *)&tempstr);
       if (tempstr==NULL) { (*PlotItemPtr)->label = NULL; }
       else
        {
@@ -1681,16 +1720,16 @@ int directive_plot(Dict *command, int interactive, int replot)
       // Look up continuous / discontinuous flags
       (*PlotItemPtr)->ContinuitySet = 0;
       (*PlotItemPtr)->continuity    = DATAFILE_CONTINUOUS;
-      DictLookup(TempDict, "continuous", NULL, (void **)&tempstr);
+      DictLookup(TempDict, "continuous", NULL, (void *)&tempstr);
       if (tempstr!=NULL) { (*PlotItemPtr)->ContinuitySet = 1; (*PlotItemPtr)->continuity = DATAFILE_CONTINUOUS; }
-      DictLookup(TempDict, "discontinuous", NULL, (void **)&tempstr);
+      DictLookup(TempDict, "discontinuous", NULL, (void *)&tempstr);
       if (tempstr!=NULL) { (*PlotItemPtr)->ContinuitySet = 1; (*PlotItemPtr)->continuity = DATAFILE_DISCONTINUOUS; }
 
       // Look up title and notitle modifiers
       (*PlotItemPtr)->NoTitleSet = (*PlotItemPtr)->TitleSet = 0; (*PlotItemPtr)->title = NULL;
-      DictLookup(TempDict, "notitle", NULL, (void **)&tempstr);
+      DictLookup(TempDict, "notitle", NULL, (void *)&tempstr);
       if (tempstr!=NULL) { (*PlotItemPtr)->NoTitleSet = 1; (*PlotItemPtr)->TitleSet = 0; (*PlotItemPtr)->title = NULL; }
-      DictLookup(TempDict, "title", NULL, (void **)&tempstr);
+      DictLookup(TempDict, "title", NULL, (void *)&tempstr);
       if (tempstr!=NULL)
        {
         (*PlotItemPtr)->NoTitleSet = 0; (*PlotItemPtr)->TitleSet = 1;
@@ -1702,11 +1741,11 @@ int directive_plot(Dict *command, int interactive, int replot)
       // Look up index , every, using modifiers
       (*PlotItemPtr)->UsingRowCols = DATAFILE_COL;
       (*PlotItemPtr)->index = -1;
-      DictLookup(TempDict, "index"      , NULL, (void **)&indexptr);   if (((*PlotItemPtr)->IndexSet = (indexptr!=NULL))==1) (*PlotItemPtr)->index = *indexptr;
-      DictLookup(TempDict, "use_rows"   , NULL, (void **)&tempstr);    if (tempstr  != NULL) (*PlotItemPtr)->UsingRowCols = DATAFILE_ROW;
-      DictLookup(TempDict, "use_cols"   , NULL, (void **)&tempstr);    if (tempstr  != NULL) (*PlotItemPtr)->UsingRowCols = DATAFILE_COL;
+      DictLookup(TempDict, "index"      , NULL, (void *)&indexptr);   if (((*PlotItemPtr)->IndexSet = (indexptr!=NULL))==1) (*PlotItemPtr)->index = *indexptr;
+      DictLookup(TempDict, "use_rows"   , NULL, (void *)&tempstr);    if (tempstr  != NULL) (*PlotItemPtr)->UsingRowCols = DATAFILE_ROW;
+      DictLookup(TempDict, "use_cols"   , NULL, (void *)&tempstr);    if (tempstr  != NULL) (*PlotItemPtr)->UsingRowCols = DATAFILE_COL;
 
-      DictLookup(TempDict, "every_list:", NULL, (void **)&EveryList);
+      DictLookup(TempDict, "every_list:", NULL, (void *)&EveryList);
       if (EveryList==NULL) { (*PlotItemPtr)->EverySet = 0; }
       else
        {
@@ -1716,7 +1755,7 @@ int directive_plot(Dict *command, int interactive, int replot)
         for (i=0; i<j; i++)
          {
           TempDict2 = (Dict *)ListIter2->data;
-          DictLookup(TempDict2, "every_item", NULL, (void **)&tempint);
+          DictLookup(TempDict2, "every_item", NULL, (void *)&tempint);
           if (tempint != NULL) (*PlotItemPtr)->EveryList[i] = *tempint;
           else                 (*PlotItemPtr)->EveryList[i] = -1;
           ListIter2 = ListIterate(ListIter2, NULL);
@@ -1724,7 +1763,7 @@ int directive_plot(Dict *command, int interactive, int replot)
        }
 
       // Look up select modifier
-      DictLookup(TempDict, "select_criterion", NULL, (void **)&SelectCrit);
+      DictLookup(TempDict, "select_criterion", NULL, (void *)&SelectCrit);
       if (SelectCrit==NULL) { (*PlotItemPtr)->SelectCriterion = NULL; }
       else
        {
@@ -1734,7 +1773,7 @@ int directive_plot(Dict *command, int interactive, int replot)
        }
 
       // Look up using modifiers
-      DictLookup(TempDict, "using_list:", NULL, (void **)&UsingList);
+      DictLookup(TempDict, "using_list:", NULL, (void *)&UsingList);
       if (UsingList==NULL) { (*PlotItemPtr)->NUsing = 0; (*PlotItemPtr)->UsingList = NULL; }
       else
        {
@@ -1743,7 +1782,7 @@ int directive_plot(Dict *command, int interactive, int replot)
          {
           TempDict2 = (Dict *)(UsingList->first->data);
           if (TempDict2==NULL) { (*PlotItemPtr)->NUsing = 0; (*PlotItemPtr)->UsingList = NULL; goto FinishedUsing; }
-          DictLookup(TempDict2, "using_item", NULL, (void **)&cptr2);
+          DictLookup(TempDict2, "using_item", NULL, (void *)&cptr2);
           if (cptr2==NULL) { (*PlotItemPtr)->NUsing = 0; (*PlotItemPtr)->UsingList = NULL; goto FinishedUsing; }
          }
         (*PlotItemPtr)->NUsing = j;
@@ -1753,7 +1792,7 @@ int directive_plot(Dict *command, int interactive, int replot)
         for (i=0; i<j; i++)
          {
           TempDict2 = (Dict *)ListIter2->data;
-          DictLookup(TempDict2, "using_item", NULL, (void **)&cptr2);
+          DictLookup(TempDict2, "using_item", NULL, (void *)&cptr2);
           if (cptr2==NULL) cptr2=""; // NULL expression means blank using expression
           (*PlotItemPtr)->UsingList[i] = (char *)malloc(strlen(cptr2)+1);
           if ((*PlotItemPtr)->UsingList[i] == NULL)  { ppl_error(ERR_MEMORY, -1, -1,"Out of memory."); free(*PlotItemPtr); *PlotItemPtr = NULL; return 1; }
@@ -1766,6 +1805,38 @@ FinishedUsing:
       // Read in style information from with clause
       with_words_fromdict(TempDict, &(*PlotItemPtr)->ww, 1);
 
+      // If plotting data from pipe, read and store it now
+      if (((*PlotItemPtr)->filename!=NULL) && ((strcmp((*PlotItemPtr)->filename,"-")==0) || (strcmp((*PlotItemPtr)->filename,"--")==0)))
+       {
+        canvas_plotdesc *pd = (*PlotItemPtr);
+        int              j, status=0, ErrCount, NExpect, linespoints;
+        List            *UsingList, *EveryList;
+        Dict            *tempdict;
+        unsigned char    AutoUsingList=0;
+        char            *errbuff;
+
+        UsingList   = ListInit(); for (j=0; j<pd->NUsing  ; j++) { tempdict = DictInit(HASHSIZE_SMALL); DictAppendPtr(tempdict, "using_item", (void *)pd->UsingList[j], 0, 0, DATATYPE_VOID); ListAppendPtr(UsingList, (void *)tempdict, 0, 0, DATATYPE_VOID); }
+        EveryList   = ListInit(); for (j=0; j<pd->EverySet; j++) { tempdict = DictInit(HASHSIZE_SMALL); DictAppendPtr(tempdict, "every_item", (void *)(pd->EveryList+j), 0, 0, DATATYPE_VOID); ListAppendPtr(EveryList, (void *)tempdict, 0, 0, DATATYPE_VOID); }
+        ErrCount    = DATAFILE_NERRS;
+        linespoints = pd->ww.USElinespoints ? pd->ww.linespoints : (ptr->settings.DataStyle.USElinespoints ? ptr->settings.DataStyle.linespoints : SW_STYLE_POINTS);
+        NExpect     = eps_plot_styles_NDataColumns(linespoints, ptr->ThreeDim);
+
+        pd->ww.linespoints    = linespoints; // Fix plot style, so that number of expected columns doesn't later change with DataStyle
+        pd->ww.USElinespoints = 1;
+
+        if (linespoints==SW_STYLE_COLOURMAP)
+         {
+          int ll = ListLen(UsingList);
+          if ((ll>=3)&&(ll<=6)) NExpect=ll; // Colour maps can take 3,4,5 or 6 columns of data
+         }
+
+        errbuff = (char *)lt_malloc(LSTR_LENGTH);
+        if (errbuff==NULL) { ppl_error(ERR_MEMORY, -1, -1,"Out of memory."); free(*PlotItemPtr); *PlotItemPtr = NULL; return 1; }
+
+        if (eps_plot_AddUsingItemsForWithWords(&pd->ww, &NExpect, &AutoUsingList, UsingList)) { free(*PlotItemPtr); *PlotItemPtr = NULL; return 1; } // Add extra using items for, e.g. "linewidth $3".
+        DataFile_read(&pd->PersistentDataTable, &status, errbuff, pd->filename, pd->index, pd->UsingRowCols, UsingList, AutoUsingList, EveryList, pd->label, NExpect, pd->SelectCriterion, pd->continuity, (linespoints==SW_STYLE_BOXES)?"@":NULL, DATAFILE_DISCONTINUOUS, 1, &ErrCount);
+       }
+
       PlotItemPtr=&(*PlotItemPtr)->next; // Next plot item...
       if (!PlottingDatafiles) break; // Only loop once when plotting functions, as there's no globbing to do
      }
