@@ -3,10 +3,10 @@
 // The code in this file is part of PyXPlot
 // <http://www.pyxplot.org.uk>
 //
-// Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
-//               2008-2010 Ross Church
+// Copyright (C) 2006-2011 Dominic Ford <coders@pyxplot.org.uk>
+//               2008-2011 Ross Church
 //
-// $Id: ppl_canvasitems.h 863 2010-07-02 20:00:22Z dcf21 $
+// $Id: ppl_canvasitems.h 976 2010-11-23 15:02:24Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -47,13 +47,16 @@ typedef struct canvas_plotrange {
 } canvas_plotrange;
 
 typedef struct canvas_plotdesc {
- unsigned char           function, parametric, TRangeSet, axis1set, axis2set, axis3set, ContinuitySet, IndexSet, EverySet, TitleSet, NoTitleSet;
+ unsigned char           function, parametric, TRangeSet, VRangeSet, axis1set, axis2set, axis3set, ContinuitySet, IndexSet, EverySet, TitleSet, NoTitleSet;
  int                     NFunctions, axis1xyz, axis2xyz, axis3xyz, axis1, axis2, axis3, EveryList[6], index, continuity, UsingRowCols, NUsing;
  with_words              ww;
  char                   *filename, **functions, *label, *SelectCriterion, *title, **UsingList;
- value                   Tmin, Tmax;
+ value                   Tmin, Tmax, Vmin, Vmax;
  struct canvas_plotdesc *next;
 
+ // used with plot '-' and plot '--' to indicate that data is read only once; otherwise NULL
+ DataTable              *PersistentDataTable;
+
  // Structure members which are used at plot time
  DataTable              *data;
  with_words              ww_final;
@@ -64,6 +67,7 @@ typedef struct canvas_plotdesc {
  unsigned char           CRangeDisplay;
  double                  CMinFinal, CMaxFinal;
  settings_axis           C1Axis;
+ int                     GridXSize, GridYSize;
  double                  PieChart_total;
 } canvas_plotdesc;
 
