@@ -3,10 +3,10 @@
 // The code in this file is part of PyXPlot
 // <http://www.pyxplot.org.uk>
 //
-// Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
-//               2008-2010 Ross Church
+// Copyright (C) 2006-2011 Dominic Ford <coders@pyxplot.org.uk>
+//               2008-2011 Ross Church
 //
-// $Id: ppl_children.c 827 2010-06-08 23:02:30Z dcf21 $
+// $Id: ppl_children.c 974 2010-11-13 20:58:47Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -192,7 +192,7 @@ void CSPCheckForChildExits(int signo)
   iter = ListIterateInit(GhostViews);
   while (iter != NULL)
    {
-    iter = ListIterate(iter, (void **)&gv_pid);
+    iter = ListIterate(iter, (void *)&gv_pid);
     if (waitpid(*gv_pid,NULL,WNOHANG) != 0)
      {
       if (DEBUG) { sprintf(temp_err_string, "A ghostview process with pid %d has terminated.", *gv_pid); ppl_log(temp_err_string); }
@@ -203,7 +203,7 @@ void CSPCheckForChildExits(int signo)
   iter = ListIterateInit(GhostView_Persists);
   while (iter != NULL)
    {
-    iter = ListIterate(iter, (void **)&gv_pid);
+    iter = ListIterate(iter, (void *)&gv_pid);
     if (waitpid(*gv_pid,NULL,WNOHANG) != 0)
      {
       if (DEBUG) { sprintf(temp_err_string, "A persistent ghostview process with pid %d has terminated.", *gv_pid); ppl_log(temp_err_string); }
@@ -388,7 +388,7 @@ void CSPKillAllGvs()
   iter = ListIterateInit(GhostViews);
   while (iter != NULL)
    {
-    iter = ListIterate(iter, (void **)&gv_pid);
+    iter = ListIterate(iter, (void *)&gv_pid);
     kill(*gv_pid, SIGTERM); // Dulce et decorum est pro patria mori
     if (GhostView_pid == *gv_pid) GhostView_pid = 0;
    }
@@ -421,7 +421,7 @@ void PPLCheckForChildExits(int signo)
   iter = ListIterateInit(HelperPIDs);
   while (iter != NULL)
    {
-    iter = ListIterate(iter, (void **)&pid);
+    iter = ListIterate(iter, (void *)&pid);
     if (waitpid(*pid,NULL,WNOHANG) != 0)
      {
       if (DEBUG) { sprintf(text, "A helper process with pid %d has terminated.", *pid); ppl_log(text); }
@@ -443,7 +443,7 @@ void PPLKillAllHelpers()
   iter = ListIterateInit(HelperPIDs);
   while (iter != NULL)
    {
-    iter = ListIterate(iter, (void **)&pid);
+    iter = ListIterate(iter, (void *)&pid);
     kill(*pid, SIGTERM); // Dulce et decorum est pro patria mori
    }
   sigprocmask(SIG_UNBLOCK, &sigs, NULL);
