@@ -3,10 +3,10 @@
 // The code in this file is part of PyXPlot
 // <http://www.pyxplot.org.uk>
 //
-// Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
-//               2008-2010 Ross Church
+// Copyright (C) 2006-2011 Dominic Ford <coders@pyxplot.org.uk>
+//               2008-2011 Ross Church
 //
-// $Id: ppl_datafile.c 936 2010-08-27 19:38:12Z dcf21 $
+// $Id: ppl_datafile.c 977 2010-11-23 15:26:05Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -638,7 +638,7 @@ void DataFile_RotateRawData(RawDataTable **in, DataTable *out, char **UsingItems
 // DataFile_read is the main entry point for reading a table of data from a data file
 // ----------------------------------------------------------------------------------
 
-void DataFile_read(DataTable **output, int *status, char *errout, char *filename, int index, int UsingRowCol, List *UsingList, unsigned char AutoUsingList, List *EveryList, char *LabelStr, int Ncolumns, char *SelectCriterion, int continuity, char *SortBy, int SortByContinuity, int *ErrCounter)
+void DataFile_read(DataTable **output, int *status, char *errout, char *filename, int index, int UsingRowCol, List *UsingList, unsigned char AutoUsingList, List *EveryList, char *LabelStr, int Ncolumns, char *SelectCriterion, int continuity, char *SortBy, int SortByContinuity, unsigned char persistent, int *ErrCounter)
  {
   unsigned char HadNonNullUsingItem=0, ReadFromCommandLine=0, discontinuity=0, hadwhitespace, hadcomma, OneColumnInput=1;
   int           UsingLen, logi, logj, ContextOutput, ContextRough, ContextRaw;
@@ -675,7 +675,7 @@ void DataFile_read(DataTable **output, int *status, char *errout, char *filename
    {
     if (UsingLen>=USING_ITEMS_MAX) { strcpy(errout,"Too many items in using list."); *status=1; if (DEBUG) ppl_log(errout); return; }
     tempdict = (Dict *)listiter->data;
-    DictLookup(tempdict,"using_item",NULL,(void **)&(UsingItems[UsingLen]));
+    DictLookup(tempdict,"using_item",NULL,(void *)&(UsingItems[UsingLen]));
     if ((UsingItems[UsingLen]==NULL)||(strlen(StrStrip(UsingItems[UsingLen], UsingItems[UsingLen]))==0)) // Using item is blank; replace with item number
      {
       if ((UsingItems[UsingLen] = (char *)lt_malloc(10))==NULL) { sprintf(errout,"Out of memory."); *status=1; if (DEBUG) ppl_log(errout); return; };
@@ -727,22 +727,22 @@ void DataFile_read(DataTable **output, int *status, char *errout, char *filename
   // Read items out of Every List
   listiter = ListIterateInit(EveryList);
   if   (listiter == NULL) { linestep   =  1; } // READ linestep
-  else                    { tempdict = (Dict *)listiter->data; listiter = ListIterate(listiter, NULL); DictLookup(tempdict,"every_item",NULL,(void **)&intptr);
+  else                    { tempdict = (Dict *)listiter->data; listiter = ListIterate(listiter, NULL); DictLookup(tempdict,"every_item",NULL,(void *)&intptr);
                             if ((intptr==NULL) || (*intptr<1)) { linestep  =  1; } else { linestep   = *intptr; } }
   if   (listiter == NULL) { blockstep  =  1; } // READ blockstep
-  else                    { tempdict = (Dict *)listiter->data; listiter = ListIterate(listiter, NULL); DictLookup(tempdict,"every_item",NULL,(void **)&intptr);
+  else                    { tempdict = (Dict *)listiter->data; listiter = ListIterate(listiter, NULL); DictLookup(tempdict,"every_item",NULL,(void *)&intptr);
                             if ((intptr==NULL) || (*intptr<1)) { blockstep =  1; } else { blockstep  = *intptr; } }
   if   (listiter == NULL) { linefirst  = -1; } // READ linefirst
-  else                    { tempdict = (Dict *)listiter->data; listiter = ListIterate(listiter, NULL); DictLookup(tempdict,"every_item",NULL,(void **)&intptr);
+  else                    { tempdict = (Dict *)listiter->data; listiter = ListIterate(listiter, NULL); DictLookup(tempdict,"every_item",NULL,(void *)&intptr);
                             if ((intptr==NULL) || (*intptr<0)) { linefirst = -1; } else { linefirst  = *intptr; } }
   if   (listiter == NULL) { blockfirst = -1; } // READ blockfirst
-  else                    { tempdict = (Dict *)listiter->data; listiter = ListIterate(listiter, NULL); DictLookup(tempdict,"every_item",NULL,(void **)&intptr);
+  else                    { tempdict = (Dict *)listiter->data; listiter = ListIterate(listiter, NULL); DictLookup(tempdict,"every_item",NULL,(void *)&intptr);
                             if ((intptr==NULL) || (*intptr<0)) { blockfirst= -1; } else { blockfirst = *intptr; } }
   if   (listiter == NULL) { linelast   = -1; } // READ linelast
-  else                    { tempdict = (Dict *)listiter->data; listiter = ListIterate(listiter, NULL); DictLookup(tempdict,"every_item",NULL,(void **)&intptr);
+  else                    { tempdict = (Dict *)listiter->data; listiter = ListIterate(listiter, NULL); DictLookup(tempdict,"every_item",NULL,(void *)&intptr);
                             if ((intptr==NULL) || (*intptr<0)) { linelast  = -1; } else { linelast   = *intptr; } }
   if   (listiter == NULL) { blocklast  = -1; } // READ blocklast
-  else                    { tempdict = (Dict *)listiter->data; listiter = ListIterate(listiter, NULL); DictLookup(tempdict,"every_item",NULL,(void **)&intptr);
+  else                    { tempdict = (Dict *)listiter->data; listiter = ListIterate(listiter, NULL); DictLookup(tempdict,"every_item",NULL,(void *)&intptr);
                             if ((intptr==NULL) || (*intptr<0)) { blocklast = -1; } else { blocklast  = *intptr; } }
   if   (listiter != NULL) { strcpy(errout, "More than six items specified in every modifier -- final items are not valid syntax."); *status=1; if (DEBUG) ppl_log(errout); return; }
 
@@ -780,7 +780,7 @@ void DataFile_read(DataTable **output, int *status, char *errout, char *filename
  }
 
   // Keep a record of the memory context we're going to output into, and then make a scratchpad context
-  ContextOutput = lt_GetMemContext();
+  ContextOutput = persistent ? 0 : lt_GetMemContext();
   ContextRough  = lt_DescendIntoNewContext(); // Rough mallocs inside this subroutine happen in this context
   ContextRaw    = lt_DescendIntoNewContext(); // Raw data goes into here when plotting with rows
 
@@ -1060,7 +1060,7 @@ void DataFile_FromFunctions(double *OrdinateRaster, unsigned char FlagParametric
    {
     if (UsingLen>=USING_ITEMS_MAX) { strcpy(errout,"Too many items in using list."); *status=1; if (DEBUG) ppl_log(errout); return; }
     tempdict = (Dict *)listiter->data;
-    DictLookup(tempdict,"using_item",NULL,(void **)&(UsingItems[UsingLen]));
+    DictLookup(tempdict,"using_item",NULL,(void *)&(UsingItems[UsingLen]));
     if ((UsingItems[UsingLen]==NULL)||(strlen(StrStrip(UsingItems[UsingLen], UsingItems[UsingLen]))==0)) // Using item is blank; replace with item number
      {
       if ((UsingItems[UsingLen] = (char *)lt_malloc(10))==NULL) { sprintf(errout,"Out of memory."); *status=1; if (DEBUG) ppl_log(errout); return; };
@@ -1126,22 +1126,11 @@ void DataFile_FromFunctions(double *OrdinateRaster, unsigned char FlagParametric
   OrdinateVar2 = NULL;
   for (a=0; a<=SampleGrid; a++)
    {
-    char *v; value **_OrdinateVar , *_DummyTemp, **_RasterUnits;
-    if (!a) { v="x"; _OrdinateVar=&OrdinateVar ; _DummyTemp=&DummyTemp ; _RasterUnits=&RasterUnits ; }
-    else    { v="y"; _OrdinateVar=&OrdinateVar2; _DummyTemp=&DummyTemp2; _RasterUnits=&RasterYUnits; }
+    char *v,*v2; value **_OrdinateVar , *_DummyTemp, **_RasterUnits;
+    if (!a) { v="x"; v2=SampleGrid?"u":"t"; _OrdinateVar=&OrdinateVar ; _DummyTemp=&DummyTemp ; _RasterUnits=&RasterUnits ; }
+    else    { v="y"; v2="v";                _OrdinateVar=&OrdinateVar2; _DummyTemp=&DummyTemp2; _RasterUnits=&RasterYUnits; }
 
-    DictLookup(_ppl_UserSpace_Vars, FlagParametric?"t":v, NULL, (void **)_OrdinateVar);
-    if (*_OrdinateVar!=NULL)
-     {
-      *_DummyTemp = **_OrdinateVar;
-     }
-    else // If variable is not defined, create it now
-     {
-      ppl_units_zero(_DummyTemp);
-      DictAppendValue(_ppl_UserSpace_Vars, FlagParametric?"t":v, *_DummyTemp);
-      DictLookup(_ppl_UserSpace_Vars, FlagParametric?"t":v, NULL, (void **)_OrdinateVar);
-      (*_DummyTemp).modified = 2;
-     }
+    ppl_UserSpace_GetVarPointer(FlagParametric?v2:v, _OrdinateVar, _DummyTemp);
     **_OrdinateVar = **_RasterUnits;
     (*_OrdinateVar)->imag        = 0.0;
     (*_OrdinateVar)->FlagComplex = 0;
@@ -1150,20 +1139,20 @@ void DataFile_FromFunctions(double *OrdinateRaster, unsigned char FlagParametric
   // Loop over ordinate values
   {
    int i, ilen = RasterLen, i2, i2len = SampleGrid ? RasterYLen : 1; long p=0;
-   for (i2=0; i2<i2len; i2++) for (i=0; i<ilen; i++, p++)
+   for (i2=0; i2<i2len; i2++,discontinuity=1) for (i=0; i<ilen; i++, p++)
    {
     OrdinateVar->real = OrdinateRaster[i];
     if (SampleGrid) OrdinateVar2->real = OrdinateYRaster[i2];
-    if (SampleGrid) sprintf(buffer, "x=%s; y=%s", NumericDisplay(OrdinateVar->real, 0, settings_term_current.SignificantFigures, 0), NumericDisplay(OrdinateVar2->real, 1, settings_term_current.SignificantFigures, 0));
+    if (SampleGrid) sprintf(buffer, "%c=%s; %c=%s", (FlagParametric?'u':'x'), NumericDisplay(OrdinateVar->real, 0, settings_term_current.SignificantFigures, 0), (FlagParametric?'v':'y'), NumericDisplay(OrdinateVar2->real, 1, settings_term_current.SignificantFigures, 0));
     else            sprintf(buffer, "%c=%s", (FlagParametric?'t':'x'), NumericDisplay(OrdinateVar->real, 0, settings_term_current.SignificantFigures, 0));
     ppl_units_zero(ColumnData_val+0);
     ColumnData_val[0].real = p;
-    if (!FlagParametric) ColumnData_val[1] = *OrdinateVar;
-    if (SampleGrid     ) ColumnData_val[2] = *OrdinateVar2;
+    if  (!FlagParametric)              ColumnData_val[1] = *OrdinateVar;
+    if ((!FlagParametric)&&SampleGrid) ColumnData_val[2] = *OrdinateVar2;
     for (j=0; j<fnlist_len; j++)
      {
       *status=-1; k=-1;
-      ppl_EvaluateAlgebra(fnlist[j], ColumnData_val+j+1+(!FlagParametric)+SampleGrid, 0, &k, 0, status, errout, 0);
+      ppl_EvaluateAlgebra(fnlist[j], ColumnData_val+j+1+(!FlagParametric)+((!FlagParametric)&&SampleGrid), 0, &k, 0, status, errout, 0);
       if (k<strlen(fnlist[j])) { sprintf(errout, "Expression '%s' is not syntactically valid %d %ld", fnlist[j],k,(long)strlen(fnlist[j])); *status=1; if (DEBUG) ppl_log(errout); return; }
       if (*status>=0) { COUNTEDERR1; sprintf(temp_err_string, "%s: Could not evaluate expression <%s>. The error, encountered at character position %d, was: '%s'", buffer, fnlist[j], *status, errout); ppl_error(ERR_NUMERIC, -1, -1, temp_err_string); COUNTEDERR2F; *status=1; break; }
       else            { *status=0; }
@@ -1171,7 +1160,7 @@ void DataFile_FromFunctions(double *OrdinateRaster, unsigned char FlagParametric
     ExpectedNrows = (*output)->Nrows+1;
     if (!(*status))
      {
-      DataFile_ApplyUsingList(*output, ContextOutput, NULL, ColumnData_val, fnlist_len+(!FlagParametric)+SampleGrid, UsingItems, UsingLen, buffer, 0, NULL, i, 0, 0, DATAFILE_COL, "column", NULL, 0, NULL, 0, LabelStr, SelectCriterion, continuity, &discontinuity, ErrCounter, status, errout);
+      DataFile_ApplyUsingList(*output, ContextOutput, NULL, ColumnData_val, fnlist_len+(!FlagParametric)+((!FlagParametric)&&SampleGrid), UsingItems, UsingLen, buffer, 0, NULL, i, 0, 0, DATAFILE_COL, "column", NULL, 0, NULL, 0, LabelStr, SelectCriterion, continuity, &discontinuity, ErrCounter, status, errout);
      } else {
       if (!SampleGrid) discontinuity = 1;
      }
@@ -1188,8 +1177,8 @@ void DataFile_FromFunctions(double *OrdinateRaster, unsigned char FlagParametric
   }
 
   // Reset the variable x (and maybe y or t) to its old value
-  *OrdinateVar = DummyTemp;
-  if (OrdinateVar2!=NULL) *OrdinateVar2 = DummyTemp2;
+  ppl_UserSpace_RestoreVarPointer(&OrdinateVar, &DummyTemp);
+  if (OrdinateVar2!=NULL) ppl_UserSpace_RestoreVarPointer(&OrdinateVar2, &DummyTemp2);
 
   // If data is to be sorted, sort it now
   if ((!SampleGrid) && (SortBy != NULL))
