@@ -3,10 +3,10 @@
 // The code in this file is part of PyXPlot
 // <http://www.pyxplot.org.uk>
 //
-// Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
-//               2008-2010 Ross Church
+// Copyright (C) 2006-2011 Dominic Ford <coders@pyxplot.org.uk>
+//               2008-2011 Ross Church
 //
-// $Id: ppl_interpolation.c 882 2010-07-13 16:01:59Z dcf21 $
+// $Id: ppl_interpolation.c 979 2010-11-25 16:59:15Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -53,13 +53,13 @@
 #define COUNTEDERR1 if (ErrCount > 0) { ErrCount--;
 #define COUNTEDERR2 if (ErrCount==0) { sprintf(temp_err_string, "%s: Too many errors: no more errors will be shown.",filename); ppl_warning(ERR_STACKED, temp_err_string); } }
 
-static long int __compare_offset;
+static double *CompareZeroPoint;
 
-static int __compare(const void *x, const void *y)
+static int InterpolateSorter(const void *x, const void *y)
  {
-  if      (*(((double *)x)+__compare_offset) > *(((double *)y)+__compare_offset)) return  1.0;
-  else if (*(((double *)x)+__compare_offset) < *(((double *)y)+__compare_offset)) return -1.0;
-  else                                                                            return  0.0;
+  if      (  *(CompareZeroPoint + (*((long *)x))) > *(CompareZeroPoint + (*((long *)y)))  ) return  1;
+  else if (  *(CompareZeroPoint + (*((long *)x))) < *(CompareZeroPoint + (*((long *)y)))  ) return -1;
+  else                                                                                      return  0;
  }
 
 int directive_interpolate(Dict *command, int mode)
@@ -89,33 +89,33 @@ int directive_interpolate(Dict *command, int mode)
   gsl_interp_accel      *accel     = NULL;
 
   // Expand filename if it contains wildcards
-  DictLookup(command,"filename",NULL,(void **)(&cptr));
+  DictLookup(command,"filename",NULL,(void *)&cptr);
   if (cptr==NULL) ppl_error(ERR_INTERNAL, -1, -1, "File attribute not found in interpolate command.");
   filename = ppl_glob_oneresult(cptr);
   if (filename == NULL) return 1;
 
   status=0;
-  DictLookup(command, "fit_function",NULL, (void **)&fitfunc);    if (fitfunc  == NULL) { ppl_error(ERR_INTERNAL, -1, -1, "ppl_interpolation could not read name of function for output."); return 1; }
-  DictLookup(command, "index"      , NULL, (void **)&indexptr);   if (indexptr == NULL) indexptr = &index;
-  DictLookup(command, "use_rows"   , NULL, (void **)&tempstr);    if (tempstr  != NULL) rowcol=DATAFILE_ROW;
-  DictLookup(command, "use_cols"   , NULL, (void **)&tempstr);    if (tempstr  != NULL) rowcol=DATAFILE_COL;
-  DictLookup(command, "using_list:", NULL, (void **)&UsingList);
-  DictLookup(command, "every_list:", NULL, (void **)&EveryList);
-  DictLookup(command, "select_criterion", NULL, (void **)&SelectCrit);
+  DictLookup(command, "fit_function",NULL, (void *)&fitfunc);    if (fitfunc  == NULL) { ppl_error(ERR_INTERNAL, -1, -1, "ppl_interpolation could not read name of function for output."); return 1; }
+  DictLookup(command, "index"      , NULL, (void *)&indexptr);   if (indexptr == NULL) indexptr = &index;
+  DictLookup(command, "use_rows"   , NULL, (void *)&tempstr);    if (tempstr  != NULL) rowcol=DATAFILE_ROW;
+  DictLookup(command, "use_cols"   , NULL, (void *)&tempstr);    if (tempstr  != NULL) rowcol=DATAFILE_COL;
+  DictLookup(command, "using_list:", NULL, (void *)&UsingList);
+  DictLookup(command, "every_list:", NULL, (void *)&EveryList);
+  DictLookup(command, "select_criterion", NULL, (void *)&SelectCrit);
 
-  DictLookup(command, "range_list", NULL, (void **)&RangeList);
+  DictLookup(command, "range_list", NULL, (void *)&RangeList);
   if (RangeList == NULL) goto RANGES_DONE;
   ListIter = ListIterateInit(RangeList);
   if (ListIter == NULL) goto RANGES_DONE;
   TempDict = (Dict *)ListIter->data;
-  DictLookup(TempDict,"min",NULL,(void **)&xmin);
-  DictLookup(TempDict,"max",NULL,(void **)&xmax);
+  DictLookup(TempDict,"min",NULL,(void *)&xmin);
+  DictLookup(TempDict,"max",NULL,(void *)&xmax);
   if ((xmin!=NULL)&&(xmax!=NULL)&&(!ppl_units_DimEqual(xmin,xmax))) { sprintf(temp_err_string, "The minimum and maximum limits specified in the interpolate command for the x axis have conflicting physical dimensions. The former has units of <%s>, whilst the latter has units of <%s>.", ppl_units_GetUnitStr(xmin,NULL,NULL,0,1,0), ppl_units_GetUnitStr(xmax,NULL,NULL,1,1,0)); ppl_error(ERR_NUMERIC, -1, -1, temp_err_string); return 1; }
   ListIter = ListIterate(ListIter, NULL);
   if (ListIter == NULL) goto RANGES_DONE;
   TempDict = (Dict *)ListIter->data;
-  DictLookup(TempDict,"min",NULL,(void **)&ymin);
-  DictLookup(TempDict,"max",NULL,(void **)&ymax);
+  DictLookup(TempDict,"min",NULL,(void *)&ymin);
+  DictLookup(TempDict,"max",NULL,(void *)&ymax);
   if ((ymin!=NULL)&&(ymax!=NULL)&&(!ppl_units_DimEqual(ymin,ymax))) { sprintf(temp_err_string, "The minimum and maximum limits specified in the interpolate command for the y axis have conflicting physical dimensions. The former has units of <%s>, whilst the latter has units of <%s>.", ppl_units_GetUnitStr(ymin,NULL,NULL,0,1,0), ppl_units_GetUnitStr(ymax,NULL,NULL,1,1,0)); ppl_error(ERR_NUMERIC, -1, -1, temp_err_string); return 1; }
   ListIter = ListIterate(ListIter, NULL);
   if (ListIter != NULL) { ppl_error(ERR_SYNTAX, -1, -1, "Too many ranges have been supplied to the interpolate command. Only two are allowed: one for each ordinate."); return 1; }
@@ -161,7 +161,7 @@ RANGES_DONE:
   // Read input data
   if (bmp<0)
    {
-    DataFile_read(&data, &status, errtext, filename, *indexptr, rowcol, UsingList, 0, EveryList, NULL, NcolRequired, SelectCrit, continuity, NULL, -1, &ErrCount);
+    DataFile_read(&data, &status, errtext, filename, *indexptr, rowcol, UsingList, 0, EveryList, NULL, NcolRequired, SelectCrit, continuity, NULL, -1, 0, &ErrCount);
 
     // Transfer data from multiple data tables into single vectors
     if ((NcolRequired<3)&&(mode!=INTERP_STEPWISE)) xdata = (double *)lt_malloc_incontext(NcolRequired * (data->Nrows+2) * sizeof(double), ContextLocalVec);
@@ -274,10 +274,32 @@ RANGES_DONE:
     lt_AscendOutOfContext(ContextDataTab);
 
     // Sort data vectors according to x values
-    __compare_offset = -Nrows;
-    qsort((void *)ydata, i, sizeof(double), __compare);
-    __compare_offset = 0;
-    qsort((void *)xdata, i, sizeof(double), __compare);
+    {
+     long int *SortArray = (long int *)lt_malloc(2 * i * sizeof(long int));
+     long int *SortArrayI= SortArray + i;
+     long int j;
+     if (SortArray==NULL) { ppl_error(ERR_MEMORY, -1, -1, "Out of memory."); return 1; }
+     for (j=0; j<i; j++) SortArray[j]=j;
+
+     CompareZeroPoint = xdata;
+     qsort((void *)SortArray, i, sizeof(long int), InterpolateSorter);
+     for (j=0; j<i; j++) SortArrayI[SortArray[j]]=j;
+
+     for (j=0; j<i; j++)
+      {
+       double tmp,tmp2; long int tmp3;
+       tmp3            = SortArray[  j ];
+       if (tmp3==j) continue;
+       tmp             = xdata    [  j ];
+       xdata    [  j ] = xdata    [tmp3];
+       xdata    [tmp3] = tmp;
+       tmp2            = ydata    [  j ];
+       ydata    [  j ] = ydata    [tmp3];
+       ydata    [tmp3] = tmp2;
+       SortArray[SortArrayI[j]] = tmp3;
+       SortArrayI[tmp3]         = SortArrayI[j];
+      }
+    }
 
     // Filter out repeat values of x
     for (j=k=0; j<i; j++)
