@@ -3,10 +3,10 @@
 // The code in this file is part of PyXPlot
 // <http://www.pyxplot.org.uk>
 //
-// Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
-//               2008-2010 Ross Church
+// Copyright (C) 2006-2011 Dominic Ford <coders@pyxplot.org.uk>
+//               2008-2011 Ross Church
 //
-// $Id: ppl_texify.c 948 2010-09-04 17:25:15Z dcf21 $
+// $Id: ppl_texify.c 975 2010-11-16 16:49:35Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -166,7 +166,7 @@ void texify_quotedstring(char *in, int *end, char *out, int EvalStrings, int *st
     else if (in[i]=='{' ) { out[j++]='\\'; out[j++]=in[i]; }
     else if (in[i]=='}' ) { out[j++]='\\'; out[j++]=in[i]; }
     else if (in[i]=='#' ) { out[j++]='\\'; out[j++]=in[i]; }
-    else if (in[i]=='^' ) { strcpy(out+j, "\\^{}"); j+=strlen(out+j); }
+    else if (in[i]=='^' ) { strcpy(out+j, "\\verb|^|"); j+=strlen(out+j); }
     else if (in[i]=='~' ) { strcpy(out+j, "$\\sim$"); j+=strlen(out+j); }
     else if (in[i]=='<' ) { strcpy(out+j, "$<$"); j+=strlen(out+j); }
     else if (in[i]=='>' ) { strcpy(out+j, "$>$"); j+=strlen(out+j); }
@@ -452,7 +452,7 @@ void texify_algebra(char *in, int *end, char *out, int EvalStrings, int *status,
       else if ((in[i]=='|')                                ) strcpy(out+outpos, "|");
       else if ((in[i]=='<')                                ) strcpy(out+outpos, "< ");
       else if ((in[i]=='>')                                ) strcpy(out+outpos, "> ");
-      else if ((in[i]=='^')                                ) strcpy(out+outpos, "\\^{}");
+      else if ((in[i]=='^')                                ) strcpy(out+outpos, "\\wedge ");
       outpos += strlen(out+outpos);
       while (StatusRow[i]==7) i++; i--;
       LastItemContainedSuperscript = 0;
@@ -511,9 +511,20 @@ void texify_algebra(char *in, int *end, char *out, int EvalStrings, int *status,
         StrBracketMatch(in+j, CommaPositions, &Nargs, &k, MAX_STR_FORMAT_ITEMS);
         if ((RequiredNargs >= 0) && (RequiredNargs != Nargs))
          {
-          *status=j;
-          sprintf(errtext,"The %s() function takes %d arguments, but %d have been supplied in expression to be texified.", FunctionName, RequiredNargs, Nargs);
-          return;
+          int exception = 0;
+          if ((RequiredNargs==0)&&(Nargs==1))
+           {
+            int i;
+            exception=1;
+            for (i=CommaPositions[0]+1; i<CommaPositions[1]; i++) if (!((in[j+i]>='\0')&&(in[j+i]<=' '))) exception=0;
+            if (exception) Nargs=0;
+           }
+          if (!exception)
+           {
+            *status=j;
+            sprintf(errtext,"The %s() function takes %d arguments, but %d have been supplied in expression to be texified.", FunctionName, RequiredNargs, Nargs);
+            return;
+           }
          }
 
         // Realise LaTeX model
