@@ -3,10 +3,10 @@
 // The code in this file is part of PyXPlot
 // <http://www.pyxplot.org.uk>
 //
-// Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
-//               2008-2010 Ross Church
+// Copyright (C) 2006-2011 Dominic Ford <coders@pyxplot.org.uk>
+//               2008-2011 Ross Church
 //
-// $Id: ppl_userspace.h 886 2010-07-14 16:21:46Z dcf21 $
+// $Id: ppl_userspace.h 969 2010-10-14 12:45:29Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -94,13 +94,15 @@ extern Dict *_ppl_UserSpace_Funcs;
 extern Dict *_ppl_UserSpace_Funcs2;
 #endif
 
-void ppl_UserSpaceInit          ();
-void ppl_UserSpace_SetVarStr    (char *name, char   *inval, int modified);
-void ppl_UserSpace_SetVarNumeric(char *name, value  *inval, int modified);
-void ppl_UserSpace_UnsetVar     (char *name);
-void ppl_UserSpace_SetFunc      (char *definition, int modified, int *status, char *errtext);
-void ppl_UserSpace_FuncDestroy  (FunctionDescriptor *in);
-void ppl_UserSpace_FuncDuplicate(FunctionDescriptor *in, int modified);
+void ppl_UserSpaceInit              ();
+void ppl_UserSpace_SetVarStr        (char *name, char   *inval, int modified);
+void ppl_UserSpace_SetVarNumeric    (char *name, value  *inval, int modified);
+void ppl_UserSpace_UnsetVar         (char *name);
+void ppl_UserSpace_GetVarPointer    (char *name, value **output, value *temp);
+void ppl_UserSpace_RestoreVarPointer(value **output, value *temp);
+void ppl_UserSpace_SetFunc          (char *definition, int modified, int *status, char *errtext);
+void ppl_UserSpace_FuncDestroy      (FunctionDescriptor *in);
+void ppl_UserSpace_FuncDuplicate    (FunctionDescriptor *in, int modified);
 
 void ppl_GetQuotedString(char *in, char   *out, int start, int *end, unsigned char DollarAllowed, int *errpos, char *errtext, int RecursionDepth);
 void ppl_EvaluateAlgebra(char *in, value  *out, int start, int *end, unsigned char DollarAllowed, int *errpos, char *errtext, int RecursionDepth);
